<?php
require_once 'config.php';

/*
    VbLinkHelper   
        Parse in-app urls. Original from Android code base, port by jorge.
*/

class VbLinkHelper {
	public static $verbs = array(
		"SHOWTHREAD" => "showthread",
		"FORUMDISPLAY" => "forumdisplay",
		"FORUM" => "forum",
		"BLOG" => "blog",
		"BLOGENTRY" => "entry",
		"INDEX" => "index",
		"ARTICLE" => "content",
		"ALBUM" => "album",
		"SENDMESSAGE" => "sendmessage",
		"SEARCH" => "search",
		"LOGIN" => "login",
		"REGISTER" => "register",
		"PRIVATE" => "private",
		"PROFILE" => "profile",
		"MEMBER" => "member",
		"NEWBLOG" => "blog_post",
		"NEWTHREAD" => "newthread",
		"NEWREPLY" => "newreply",
		"SUBSCRIPTION" => "subscription",
        "EDITPOST" => "editpost",
		"ENTRY" => "entry",
		"CONTENT" => "content",
		// Probably deprecated? link.php links are getting transformed into content verbs
		"LIST" => "list",
		"NOOP" => ""
        );
        
        // this is an array of the verbs that can have more than one parameters
        //editpost came with two parameters do = updatepost and postid = <NUMBER>
        //We should add all the verbs that have more than one parameter to this array
        public static $moreThanOne = array("editpost");

	public static function parse($url) {
        $ret = new VbLink();
        $verbPat = "#vb:([a-z,_]+)#";
	$verb = '';
        if (preg_match($verbPat,$url, $concept)) {
            $verb = self::$verbs[strtoupper($concept[1])];
            $ret->setVerb(($verb != null) ? $verb : self::$verbs['NOOP']);
        }
        $paramPat = "#([a-z,A-Z,0-9]+)=([a-z,A-Z,0-9]+)#";
        $params = array();
        if(!in_array($verb, self::$moreThanOne)) {
            preg_match($paramPat,$url, $paramMatcher);
            if(!empty($paramMatcher)) {
                $params[$paramMatcher[1]] = (empty($paramMatcher[2]))? "" : $paramMatcher[2];
            }
        }
        else {
            $paramPat = "#([a-z,A-Z,0-9]+)=([a-z,A-Z,0-9]+)\/([a-z,A-Z,0-9]+)=([a-z,A-Z,0-9]+)#";
            preg_match($paramPat,$url, $paramMatcher);
            if(!empty($paramMatcher)) {
                $params = self::getMoreThanOneParameters($paramMatcher);
            }

	}
        $ret->setParameters($params);
        return $ret;
	}

    public static function translate($url) {
        $link = self::parse($url);
        $data = "";
        $urlBegin = "category_item.php";
        $urlOutOfFacebook = false;
        
        if(defined('READ_ONLY') && READ_ONLY) {
           $urlBegin = vB_SITE_ROOT . $link->getVerb() . ".php";
           $urlOutOfFacebook = true;
        } 
        switch (strtoupper($link->getVerb())) {
            case 'ALBUM':
                $parameters = $link->getParameters();
                if($urlOutOfFacebook)
                    $data = "/" . http_build_query($parameters, "", "/");
                else {
                     $albumid = (!empty($parameters["albumid"]))? "&id=" . $parameters["albumid"] : "";
                    $data = "?category=Newest_Photos" . $albumid;
                }
                break;
	    case 'CONTENT':
            case 'ARTICLE':
                $parameters = $link->getParameters();
                if($urlOutOfFacebook)
                    $data = "/" . http_build_query($parameters, "", "/");
                else {
                    $r = (!empty($parameters["r"]))? "&id=" . $parameters["r"] : "";
                    $data = "?category=Newest_Articles" . $r;
                }
                break;
	    case 'ENTRY':
            case 'BLOG':
            case 'BLOGENTRY':
                $parameters = $link->getParameters();
                $blog = $link->getFirst(array("b", "blogid", "bt", "blogtextid"));
                if($urlOutOfFacebook)
                    $data = "/" . http_build_query($parameters, "", "/");
                else {
                    $blogid = (!empty($blog))? "&id=" . $blog : "";
                    $data = "?category=Newest_Blogs" . $blogid;
                }
                break;
            case 'SHOWTHREAD':
                $thread = $link->getEither("t", "threadid");
                $data = "";
                if($urlOutOfFacebook)
                    $data = "/" . $thread;
                else {
                    $threadid = (!empty($thread))? "&id=" . $thread : "";
                    $data = "?category=Newest_Threads" . $threadid;
                }
                break;
            case 'FORUMDISPLAY':
            case 'FORUM':
                $parameters = $link->getParameters();
                $forumid = (!empty($parameters["f"]))? $parameters["f"] : "";
                if($urlOutOfFacebook)
                    $data = "/" . $forumid;
                else {
                    $urlBegin = FACEBOOK_CANVAS_URL . "category_stream.php";
                    $data = "?category=Forum_Threads_" . $forumid;
                }
                break;
            case 'PRIVATE':
                $parameters = $link->getParameters();
                $action = $parameters["do"];
                if ($action != null && $action == "newpm") {
                    $user = $link->getEither("u", "userid");
                }
                if($urlOutOfFacebook)
                    $data = "/" . http_build_query($parameters, "", "/");
                else $urlBegin = FACEBOOK_CANVAS_URL;
                break;
            case 'NEWTHREAD':
                $forum = $link->getEither("f", "forumid");
            
                if($urlOutOfFacebook){
                    $data = "/do=newthread/f=" . $forum;
                }
                else {
                    $urlBegin = FACEBOOK_CANVAS_URL . "category_stream.php";
                    $data = "?category=Forum_Threads_" . $forum;
                }
                break;
            case 'NEWREPLY':
                // TODO: Handle links with postid when we implement it.
                $thread = $link->getEither("t", "threadid");
                $post = $link->getEither("p", "postid");
                if($urlOutOfFacebook)
                    $data = "/p=" . $post . "&noquote=1";
                else
                    $data = "?category=Newest_Threads&id=" . $thread;
                break;
            case 'MEMBER':
            case 'PROFILE':
                $user = $link->getEither("u", "userid");
                if($urlOutOfFacebook)
                    $data = "/" . $user;
                else $urlBegin = FACEBOOK_CANVAS_URL;
                break;
            case 'INDEX':
            case 'SEARCH':
            case 'SUBSCRIPTION':
                $urlBegin = ($urlOutOfFacebook)? "" : FACEBOOK_CANVAS_URL;
                break;
            case 'SENDMESSAGE':
                $user = $link->getEither("u", "userid");
                if($urlOutOfFacebook)
                    $urlBegin .= "/do=mailmember/u=" . $user;
                else $urlBegin = FACEBOOK_CANVAS_URL;
                break;
            case 'LOGIN':
                if($urlOutOfFacebook)
                    $urlBegin .= "/do=login";
                else $urlBegin = FACEBOOK_CANVAS_URL;
                break;
            case 'REGISTER':
                if(!$urlOutOfFacebook)
                    $urlBegin = FACEBOOK_CANVAS_URL .  "register.php";
                break;
            case 'NEWBLOG':
                 if($urlOutOfFacebook)
                    $urlBegin .= "/do=newblog";
                else $urlBegin = FACEBOOK_CANVAS_URL;
                break;
            case 'EDITPOST':
                $parameters = $link->getParameters();
                $urlBegin = vB_SITE_ROOT . "editpost.php";
                $data = "?" . http_build_query($parameters, "", "&");
                break;
        }
        return $url = $urlBegin . $data;
    }
    
    private static function getMoreThanOneParameters($params) {
        $resultParams = array();
        $auxArray = array_slice($params,1);
        $total = count($auxArray);
        for($i=0 ; $i<=$total ; $i++) {
            if($i % 2 == 0 && array_key_exists($i, $auxArray)) {
                $resultParams[$auxArray[$i]] = $auxArray[($i+1)];
            }
        }
        return $resultParams;
    }
}

class VbLink {
    private $parameters;
    private $verb;
    public function __construct(){
        $this->verb = "";
    }
    public function getVerb()
    {
        return $this->verb;
    }

    public function setVerb($verb)
    {
        $this->verb = $verb;
    }

    public function getParameters()
    {
        return $this->parameters;
    }

    public function setParameters($parameters)
    {
        $this->parameters = $parameters;
    }

    public function getEither($a, $b)
    {
        return self::getFirst(array( $a, $b ));
    }

    public function getFirst($params)
    {
        foreach($params as $s)
        {
                $test = $this->parameters[$s];
                if(!empty($test))
                    return $test;
        }
        return null;
    }
}

?>
